# -*- encoding: utf-8 -*-
##############################################################################
#
#    Addition to OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2012 Vertel. All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
import pooler
import time
from tools.translate import _


from datetime import datetime
from dateutil.relativedelta import relativedelta

from tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, float_compare


class mdm_devices(osv.osv):
    _name = "mdm.devices"
    _inherits = {'account.analytic.account': "analytic_account_id"}
    _description = "Holds records of Devices"

    def name_get(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if not len(ids):
            return []
        result = []
        for mobile_device in self.browse(cr, uid, ids, context=context):
            result.append((mobile_device.id, '%s (%s/%s)' % (mobile_device.name,mobile_device.make,mobile_device.model)))
        return result
                
    def _complete_name(self, cr, uid, ids, name, args, context=None):
        res = {}
        for mobile_device in self.browse(cr, uid, ids, context=context):
            res[mobile_device.id] = '%s (%s/%s)' % (mobile_device.name,mobile_device.make or '',mobile_device.model or '')
        #for m in self.browse(cr, uid, ids, context=context):
            #res[m.id] = (m.parent_id and (m.parent_id.name + '/') or '') + m.name
        return res
        
    def _amount_devices(self, cr, uid, ids, field_name, arg, context=None):
        res = {}        
        for c in self.browse(cr, uid, ids, context=context):
            res[c.id] = {
                'device_cost': 0.0,
                'device_costtot': 0.0,
                'device_roi': 0.0,
            }
            contract_days = (datetime.today()-datetime.strptime(c.date_start,DEFAULT_SERVER_DATE_FORMAT)).days
            if (c.balance > 0 and contract_days > 0):
                res[c.id]['device_cost'] = round(c.balance / contract_days * 30.5,2)
                if c.date > 0:
                    contract_totdays = (datetime.strptime(c.date,DEFAULT_SERVER_DATE_FORMAT)-datetime.strptime(c.date_start,DEFAULT_SERVER_DATE_FORMAT)).days
                else:
                    contract_totdays = 365
                res[c.id]['device_costtot'] = round(c.balance / contract_days * contract_totdays,2)                 
            if (c.expcut-res[c.id]['device_cost']) > 0 and c.investment_cost > 0:
                res[c.id]['device_roi'] = round((c.expcut-res[c.id]['device_cost'])*12/c.investment_cost*100,2)
                

        return res

    def _get_parent(self, cr, uid, context=None):
        return self.pool.get('ir.model.data').browse(cr, uid,self.pool.get('ir.model.data').search(cr, uid,[('name','=','account_analytic_account_parent'),('module','=','mdm')]))[0].res_id

    def _get_user_messages(self, cr, uid, ids, field_name,arg, context=None):
        res = {}
        for id in ids:
            res[id] = []
            msg = self.pool.get('res.log').search(cr, uid, [('res_model', '=', 'mdm.devices'),('res_id','=',id)], order='create_date desc',limit=100,context=context)
            if msg:
                res[id] = msg
        return res

    _columns = {
                'complete_name': fields.function(_complete_name, string="Device Name", type='char', size=25), 
#                'name':fields.char('Device Id',help="Usually just a sequence",size=20,required=True),
                'analytic_account_id': fields.many2one('account.analytic.account', 'Analytic Account', help="Link this device to an analytic account if you need financial management. It enables you to connect devices with budgets, planning, cost and revenue analysis.", ondelete="cascade",),
                'make':fields.char('Make',size=10),
                'model':fields.char('Model',size=15),
                'colour':fields.char('Colour',size=15),
                'serial':fields.char('productSerial #',size=50),
                'class': fields.selection([
                        ('cell','Cell phone'),
                        ('smart','Smart phone'),
                        ('pad','Pad'),
                        ('other','Other'),], 'Class', required=False,),
                'os': fields.selection([
                        ('android','Android'),
                        ('ios','iOS'),
                        ('win','Windows'),
                        ('symbian','Symbian'),
                        ('rim','Rim'),
                        ('meego','Meego'),
                        ('other','Other'),], 'OS', required=False,),
                'state': fields.selection([
                        ('draft','Draft'),
                        ('active','Active'),
                        ('outofservice','Out of Service'),
                        ('deprecated','Deprecated'),
                        ], 'State', required=True,),
                'ownership': fields.selection([
                        ('owned','Owned'),
                        ('private','Private (owned by employee)'),
                        ('leased','Leased'),
                        ('by_contract','By Contract'),                       
                        ('rented','Rented'),                       
                        ], 'Ownership', required=False), 
                                        
                'pcost':fields.float('Purchase Value',digits=(16,2)),
                'warrexp':fields.date('Date',help="Expiry date for warranty of product"),
                'note':fields.text('Note'),
                'log':fields.function(_get_user_messages,string='History',method=True,type = 'one2many',relation='res.log',),
                'expcut': fields.float('Expenditure Cut',help="Budgeted expenditure cut or alternative cost per month to calculate ROI"),
                'investment_cost': fields.float('Investment Cost',help="Budgeted investment cost to calculate ROI"),
                'device_cost':fields.function(_amount_devices,string='Device Cost',type = 'float',help="Device Cost per Month",multi="device_cost"),
                'device_roi':fields.function(_amount_devices,string='Device ROI',type = 'float',help="Device Return of Investment",multi="device_roi"),
                'device_costtot':fields.function(_amount_devices,string='Device Total Cost',type = 'float',help="Device Cost during a year or end of contract",multi="device_costtot"),
                }
    _defaults={
            'state': lambda *a:'draft',
            'name': lambda obj,cr,uid,context: obj.pool.get('ir.sequence').get(cr,uid,'mdm.devices'),    
            'class':lambda *a:'smart',
            'state':lambda *a:'draft',
            'ownership':lambda *a:'owned',
            'parent_id': _get_parent,
               }

    def button_active(self, cr, uid, ids, context=None):
#        self.log(cr, uid, ids[0], "Here I come %s" % self.browse(cr,uid,ids[0]).name)
        return self.write(cr, uid, ids, {'state': 'active'})
    def button_deprecated(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'deprecated'})
    def button_outofservice(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'outofservice'})
    def button_draft(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'draft'})
        
        
        
    def on_change_partner_id(self, cr, uid, id, partner_id, context={}):
        if not partner_id:
            return {'value': {'contact_id': False}}
        addr = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['invoice'])
        return {'value': {'contact_id': addr.get('invoice', False)}}

    def on_change_company(self, cr, uid, id, company_id):
        if not company_id:
            return {}
        currency = self.pool.get('res.company').read(cr, uid, [company_id], ['currency_id'])[0]['currency_id']
        return {'value': {'currency_id': currency}}


    def onchange_partner_id(self, cr, uid, ids, partner, context=None):
        partner_obj = self.pool.get('res.partner')
        if not partner:
            return {'value':{'contact_id': False}}
        address = partner_obj.address_get(cr, uid, [partner], ['contact'])
        return {'value':{'contact_id': address['contact']}}
                


    _sql_constraints = [
        ('uniq_name', 'unique (name)', 'The name of the mobile device must be unique !')
    ]
    
mdm_devices()



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
